"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



This module is for testing the Automated Curation View Service module. We assume that the Core Content Pack
has been loaded.

The routines assume that pytest is being used as the test tool and framework.

A running Egeria environment is needed to run these tests.

"""

import json
import time
from datetime import datetime

from pydantic import ValidationError
from rich import print, print_json
from rich.console import Console
from rich.pretty import pprint

from pyegeria import  AutomatedCuration, PyegeriaException, print_basic_exception, \
    print_validation_error, PyegeriaAPIException
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)

# from pyegeria.admin_services import FullServerConfig

disable_ssl_warnings = True
console = Console(width=200)


class TestAutomatedCuration:
    good_platform1_url = "https://localhost:9443"

    good_user_1 = "garygeeke"
    good_user_2 = "erinoverview"
    good_user_3 = "peterprofile"
    bad_user_1 = "eviledna"
    bad_user_2 = ""

    good_server_1 = "simple-metadata-store"
    good_server_2 = "laz_kv"
    good_server_3 = "active-metadata-store"
    good_server_4 = "integration-daemon"

    good_engine_host_1 = "governDL01"
    good_view_server_1 = "view-server"
    good_view_server_2 = "qs-view-server"
    bad_server_1 = "coco"
    bad_server_2 = ""

    def test_create_element_from_template(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            # body = {"templateGUID": "379e6c05-9cfa-4d31-a837-0ed4eee6482a", "isOwnAnchor": "true",
            #     "placeholderPropertyValues": {"pathName": "/Users/dwolfson/localGit/datahub",
            #         "deployedImplementationType": "File Folder", "folderName": "Sample Data"}}
            # body for a omag server
            body = {
                "templateGUID": "1764a891-4234-45f1-8cc3-536af40c790d",
                "isOwnAnchor": True,
                "placeholderPropertyValues": {
                    "userId": "garygeeke",
                    "hostURL": "https://localhost",
                    "portNumber": "9446",
                    "serverName": "Survey Engine Host",
                },
            }
            start_time = time.perf_counter()
            response = a_client.create_elem_from_template(body)
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            print("Guid of created element is:" + str(response))
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_create_folder_asset(self) -> str:
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            path_name = "/deployments/loading-bay"
            folder_name = "loading-bay"
            file_system = None
            description = "Loading bay folder"
            version = "0.1"
            start_time = time.perf_counter()
            response = a_client.create_folder_element_from_template(
                path_name, folder_name, "laz.local", description, version
            )
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                pprint(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                pprint("created folder with GUID: " + response)
            assert True

        except (
            PyegeriaException, PyegeriaAPIException,
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"
        except ValidationError as e:
            print_validation_error(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_create_kafka_server_element_from_template(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            response = a_client.create_kafka_server_element_from_template(
                "pdr-kafka5", "egeria.pdr-associates.com", "9093"
            )
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                pprint(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                pprint("created Kafka Server with GUID: " + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()


    def test_create_kafka_element_from_template(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            response = a_client.create_kafka_server_element_from_template(
                "pdr-kafka5", "egeria.pdr-associates.com", "9093"
            )
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                pprint(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                pprint("created Kafka Server with GUID: " + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_create_csv_data_file_element_from_template(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()

            # response = a_client.create_csv_data_file_element_from_template(
            #     "raw emission factor data.csv",
            #     "CSV",
            #     "/deployments/loading-bay/Sustainability Files/raw emission factor data.csv",
            #     "2022-6-digit",
            #     "UTF-8",
            #     "csv",
            #     "",
            #     "NAICS was originally developed to provide a consistent framework for the collection, analysis, and dissemination of industrial statistics used by government policy analysts, by academics and researchers, by the business community, and by the public. Revisions for 2022 were made to account for our rapidly changing economies"
            # )
            #
            response = a_client.create_csv_data_file_element_from_template(
                "raw emission factor data.csv",
                "CSV",
                "/loading-bay/Sustainability Files/raw emission factor data.csv",
                None,
                "UTF-8",
                "csv",
                None,
                "Raw emissions factor data for deriving standard emissions from different fuels and sources"
            )
            # response = a_client.create_csv_data_file_element_from_template(
            #     "NAICS-6-digit_2022_Codes.csv",
            #     "CSV",
            #     "/loading-bay/Sustainability Files/NAICS-6-digit_2022_Codes.csv",
            #     "2022",
            #     "UTF-8",
            #     "csv",
            #     "",
            #     "NAICS was originally developed to provide a consistent framework for the collection, analysis, and dissemination of industrial statistics used by government policy analysts, by academics and researchers, by the business community, and by the public. Revisions for 2022 were made to account for our rapidly changing economies"
            # )

            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                pprint(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                pprint("Database Server GUID create is " + response)
            assert True

        except (
            PyegeriaException, PyegeriaAPIException
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"
        except ValidationError as e:
            print_validation_error(e)
        finally:
            a_client.close_session()

    def test_create_postgres_database_element_from_template(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            response = a_client.create_postgres_database_element_from_template(
                "egeria",
                "LocalPostgreSQL1",
                "host.docker.internal",
                "5442",
                db_user="egeria_user",
                db_pwd="user4egeria",
                description="Egeria Repository"
            )
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                pprint(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                pprint("Database Server GUID create is " + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_create_uc_server_element_from_template(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            response = a_client.create_uc_server_element_from_template(
                "laz3", "http://host.docker.internal", "8080", "my test uc", "0.1"
            )
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                pprint(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                pprint("UC Server GUID create is " + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_get_engine_actions(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()


            start_time = time.perf_counter()
            response = a_client.get_active_engine_actions()
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            print(f"The type of response is: {type(response)}")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            PyegeriaException
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"
        except ValidationError as e:
            print_validation_error(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_get_engine_action(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            engine_action_guid = "3034d5f3-2cf1-420a-8b64-103079570659"
            start_time = time.perf_counter()
            response = a_client.get_engine_action(engine_action_guid)
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is dict:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_cancel_engine_action(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            engine_action_guid = ""
            start_time = time.perf_counter()
            a_client.cancel_engine_action(engine_action_guid)
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            pprint(f"Canceled engine action: {engine_action_guid}")
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_get_active_engine_actions(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token("peterprofile","secret")

            start_time = time.perf_counter()
            response = a_client.get_active_engine_actions()
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                print(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                print("\n\n" + response)
            assert True

        except (
            PyegeriaException
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_get_engine_actions_by_name(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            response = a_client.get_engine_actions_by_name("JDBCDatabaseCataloguer")
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            print(f"\n Type of response is {type(response)}")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_find_engine_actions(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            response = a_client.find_engine_actions("*", output_format = "DICT", report_spec = "Referenceable")
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            PyegeriaException
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    #
    #   Governance Processes
    #


    def test_initiate_gov_action_process(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            action_type_qualified_name = "Egeria:DailyGovernanceActionProcess"
            start_time = time.perf_counter()
            # start_time_now: datetime = datetime.now()
            start_time_now = None
            request_source_guids = None
            action_targets = None
            request_parameters = None
            orig_service_name = None
            orig_engine_name = None

            response = a_client.initiate_gov_action_process(
                action_type_qualified_name,
                request_source_guids,
                action_targets,
                start_time_now,
                request_parameters,
                orig_service_name,
                orig_engine_name,
            )

            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            print(f"\n Type of response is {type(response)}")
            if type(response) is dict:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_get_gov_action_types_by_guid(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            gov_action_guid = ""
            start_time = time.perf_counter()
            response = a_client.get_gov_action_types_by_guid(gov_action_guid)
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            print(f"\n Type of response is {type(response)}")
            if type(response) is dict:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_get_gov_action_types_by_name(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            name = "AssetSurvey:survey-kafka-server"
            start_time = time.perf_counter()
            response = a_client.get_gov_action_types_by_name(name)
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            print(f"\n Type of response is {type(response)}")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()



    #   Get Technology types
    #

    def test_get_all_technology_types(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            response = a_client.get_all_technology_types(output_format = "DICT", report_spec = "Referenceable")
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            PyegeriaException
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_find_technology_types(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            response = a_client.find_technology_types(
                "File", starts_with=True, ignore_case=True
            )
            duration = time.perf_counter() - start_time
            print(f"\n\t# Elements was {len(response)} with {duration:.2f} seconds")
            print(f"Type of response was {type(response)}")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            PyegeriaException
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"
        except ValidationError as e:
            print_validation_error(e)

        finally:
            a_client.close_session()

    def test_get_technology_types_for_open_metadata_type(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id="peterprofile",
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            response = a_client.get_tech_types_for_open_metadata_type(
                "SoftwareServer", "Database", output_format = "JSON", report_spec = "Referenceable"
            )
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            PyegeriaException
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_get_technology_type_detail(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            # response = a_client.get_technology_type_detail("CSV Data File", True)
            response = a_client.get_technology_type_detail("File System", output_format="DICT",
                                                           report_spec="TechTypeDetail")
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)

            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is dict:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_get_template_guid_for_technology_type(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            response = a_client.get_template_guid_for_technology_type("File System",)
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is dict:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                print(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                print("\nTemplate GUID is: " + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()
    #
    #   Governance Actions
    #
    def test_initiate_gov_action_type(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            gov_action_type_qn = "Egeria:GovernanceActionType:2adeb8f1-0f59-4970-b6f2-6cc25d4d2402survey-folder"

            response = a_client.initiate_gov_action_type("PostgreSQL Server")
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is dict:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_get_catalog_target(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            postgres_server_connector_guid = "36f69fd0-54ba-4f59-8a44-11ccf2687a34"
            unity_sync_guid = "06d068d9-9e08-4e67-8c59-073bbf1013af"
            u2 = "ec069de6-5755-45cd-8b78-9c76452a060f"
            element_guid = "731eb432-e9e9-482a-86fc-0a7407ea78e6"
            rel_guid = "19a5fc39-f928-4a78-8637-ade37e0c5598"
            start_time = time.perf_counter()

            response = a_client.get_catalog_target(u2)
            duration = time.perf_counter() - start_time
            print(f"Type of response was {type(response)}")
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is dict:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_get_catalog_targets(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            postgres_server_connector_guid = a_client.get_connector_guid('PostgreSQL Server')

            element_guid = "061a4fb3-7d41-4e52-9080-65e9c61084d2"
            relationship_guid = "6be6e470-7aa2-4f50-8142-246866037523"
            # t = INTEGRATION_GUIDS['SampleDataFilesMonitor']
            t = a_client.get_connector_guid('UnityCatalogServerSynchronizer')
            # t = INTEGRATION_GUIDS["UnityCatalogServerSynchronizer"]
            start_time = time.perf_counter()

            response = a_client.get_catalog_targets(postgres_server_connector_guid)
            duration = time.perf_counter() - start_time
            print(f"Type of response was {type(response)}")
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                print(f"\n\nfor {t} got response {response} ")
            assert True

        except (
            PyegeriaException
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_add_catalog_target(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            element_guid = "ff0aec48-b3c8-4732-a0da-5ab4596c2a60"
            catalog_target_name = "exchange-folder"
            postgres_con_guid = ""
            start_time = time.perf_counter()
            connector_guid = a_client.get_connector_guid('PostgreSQLServer')
            guid = a_client.add_catalog_target(
                connector_guid, element_guid, catalog_target_name
            )
            duration = time.perf_counter() - start_time
            print(f"guid returned is: {guid}")
            print(f"\n\tDuration was {duration} seconds")
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_update_catalog_targets(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            element_guid = "001927b1-b960-48c6-a99e-73b6957c259d"
            catalog_target_name = "deltalake experiments on laz"
            file_folder_integ_con = "cd6479e1-2fe7-4426-b358-8a0cf70be117"
            relationship_guid = "2ec7af3f-1d1e-4d5d-85a4-15c62480b898"
            start_time = time.perf_counter()
            guid = a_client.update_catalog_target(
                relationship_guid, catalog_target_name
            )
            duration = time.perf_counter() - start_time
            print(f"guid returned is: {guid}")
            print(f"\n\tDuration was {duration} seconds")
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_catalog_folder_files(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            # jdbc_database_connector_guid = "70dcd0b7-9f06-48ad-ad44-ae4d7a7762aa"
            # postgres_connector_guid = "36f69fd0-54ba-4f59-8a44-11ccf2687a34"
            folder_guid = a_client.get_connector_guid("FileFolder")
            catalog_target_name = "Brain-API-Quickstart"
            file_connector_guid = a_client.get_connector_guid("FileFolder")
            # element_guid = "64296369-323f-4d74-aab3-c2ebae923d25"
            # catalog_target_name = "coco_ods_catalog_target"
            start_time = time.perf_counter()
            a_client.add_catalog_target(
                file_connector_guid, folder_guid, catalog_target_name
            )
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_remove_catalog_target(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            relationship_guid = "7d7fe5db-55bb-4f69-9e2c-e43b82692751"

            start_time = time.perf_counter()

            a_client.remove_catalog_target(relationship_guid)
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_initiate_postgres_server_survey(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            a_postgres_server_guid = "d34544cd-7cc0-4258-ad19-2c32f1f1ffed "
            start_time = time.perf_counter()

            response = a_client.initiate_postgres_server_survey(a_postgres_server_guid)
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is dict:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_initiate_postgres_database_survey(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()
            a_postgres_database_guid = "da160627-8d5d-4778-89fe-6d1df7843e30"

            start_time = time.perf_counter()

            response = a_client.initiate_postgres_database_survey(
                a_postgres_database_guid
            )
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is dict:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_initiate_file_folder_survey(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            # file_folder_guid = "58ef1911-1e85-43cc-a6cb-a8990112e591"
            file_folder_guid = "7c28d5cb-c745-4663-bdaa-19c0bf962ecf"
            response = a_client.initiate_file_folder_survey(
                file_folder_guid, "FileSurvey:survey-all-folders-and-files"
            )
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is dict:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_initiate_file_survey(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            file_guid = "9dbb47d9-4ca9-404e-b02d-2e049e6e6c6e"
            response = a_client.initiate_file_survey(file_guid)
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is dict:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_initiate_uc_server_survey(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_1,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            uc_server_guid = "527b7499-fcd0-43e9-8302-b20e8378e63d"
            response = a_client.initiate_uc_server_survey(uc_server_guid)
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is dict:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()

    def test_get_technology_type_elements(self):
        try:
            a_client = AutomatedCuration(
                self.good_view_server_2,
                self.good_platform1_url,
                user_id=self.good_user_2,
                user_pwd="secret",
            )
            token = a_client.create_egeria_bearer_token()

            start_time = time.perf_counter()
            # filter = "CSV Data File"
            filter = "PostgreSQL Server"
            response = a_client.get_technology_type_elements(
                filter, get_templates=False
            )
            duration = time.perf_counter() - start_time
            print(f"\n\tDuration was {duration} seconds")
            if type(response) is list:
                out = "\n\n" + json.dumps(response, indent=4)
                count = len(response)
                console.log(f"Found {count} elements")
                print_json(out)
            elif type(response) is str:
                console.log("\n\n" + response)
            assert True

        except (
            PyegeriaException
        ) as e:
            print_basic_exception(e)
            assert False, "Invalid request"

        finally:
            a_client.close_session()



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



This module is for testing the classification manager view service class and methods.
The routines assume that pytest is being used as the test tool and framework.

A running Egeria environment is needed to run these tests.

"""

import json
import time

from pydantic import ValidationError
from rich import print, print_json
from rich.console import Console

from pyegeria._exceptions_new import PyegeriaException, print_basic_exception, print_validation_error, \
    PyegeriaConnectionException
from pyegeria.classification_manager import ClassificationManager

# from pyegeria.output_formatter import make_preamble, make_md_attribute

disable_ssl_warnings = True

platform_url = "https://localhost:9443"
view_server = "qs-view-server"

# c_client = ClassificationManager(view_server, platform_url)

user = "erinoverview"
password = "secret"

# element_guid = '4fe24e34-490a-43f0-a0d4-fe45ac45c663'
# element_guid = "a2915132-9d9a-4449-846f-43a871b5a6a0"
# element_guid = "b359e297-a565-414a-8213-fa423312ab36" # clinical trials management
# element_guid = "25b1791f-c2fb-4b93-b236-cad53739a9a2"  # Approved Hospital
element_guid = "58933d73-7f04-4899-99ce-bbd25826041a"  # a glossary term
relationship_type = "GovernedBy"

# bearer_token = c_client.create_egeria_bearer_token(user, password)
console = Console()


def jprint(info, comment=None):
    if comment:
        print(comment)
    print(json.dumps(info, indent=2))


def valid_guid(guid):
    if (guid is None) or (type(guid) is not str):
        return False
    else:
        return True


#
##
#
def test_get_classified_elements_by():
    # metadata_element_type_name = 'CertificationType'
    #
    # metadata_element_type_name = "DeployedDatabaseSchema"
    classification_name = "confidentiality"
    try:
        c_client = ClassificationManager(view_server, platform_url)

        bearer_token = c_client.create_egeria_bearer_token(user, password)
        body = {
            "class": "LevelIdentifierQueryProperties",
            "levelIdentifier": 1
        }
        response = c_client.get_classified_elements_by(classification_name, body)

        if type(response) is list:
            print(f"\n\tElement count is: {len(response)}")
            print_json(data=response)
        elif type(response) is str:
            console.print("\n\n\t Response is: " + response)
        assert True
    except PyegeriaException as e:
        print_basic_exception(e)
        assert False, "Invalid request"
    except ValidationError as e:
        print_validation_error(e)
    except Exception as e:
        console.print_exception(show_locals=True)
        assert False, "Invalid request"
    finally:
        c_client.close_session()


def test_get_security_tagged_elements():
    # metadata_element_type_name = 'CertificationType'
    #
    # metadata_element_type_name = "DeployedDatabaseSchema"
    classification_name = "confidentiality"
    try:
        c_client = ClassificationManager(view_server, platform_url)

        bearer_token = c_client.create_egeria_bearer_token(user, password)
        body = {
            "class": "SecurityTagQueryProperties",

            "metadataElementTypeName": "GlossaryTerm",
            "limitResultsByStatus": ["ACTIVE", "DRAFT"],
            "sequencingProperty": "displayName",
            "sequencingOrder": "LAST_UPDATE_RECENT",
            "securityLabels": ["???"],
            "securityProperties": {
                "propertyName": "propertyValue"
            },
            "accessGroups": {
                "groupName": ["???"]
            }
        }
        response = c_client.get_security_tagged_elements(body, output_format="DICT", report_spec="Referenceable")

        if type(response) is list:
            print(f"\n\tElement count is: {len(response)}")
            print_json(data=response)
        elif type(response) is str:
            console.print("\n\n\t Response is: " + response)
        assert True
    except PyegeriaException as e:
        print_basic_exception(e)
        assert False, "Invalid request"
    except ValidationError as e:
        print_validation_error(e)
    except Exception as e:
        console.print_exception(show_locals=True)
        assert False, "Invalid request"
    finally:
        c_client.close_session()


def test_get_owners_elements():
    try:
        c_client = ClassificationManager(view_server, platform_url)
        bearer_token = c_client.create_egeria_bearer_token(user, password)
        owner_name = "Egeria Project"
        body = {
            "class": "FilterRequestBody",
            "filter": owner_name
        }
        response = c_client.get_owners_elements(owner_name, body, output_format="DICT",
                                                report_spec="Referenceable")

        if type(response) is list:
            print(f"\n\tElement count is: {len(response)}")
            print_json(data=response)
        elif type(response) is str:
            console.print("\n\n\t Response is: " + response)
        assert True
    except PyegeriaException as e:
        print_basic_exception(e)
        assert False, "Invalid request"
    except ValidationError as e:
        print_validation_error(e)
    except Exception as e:
        console.print_exception(show_locals=True)
        assert False, "Invalid request"
    finally:
        c_client.close_session()


def test_get_elements_by_origin():
    try:
        c_client = ClassificationManager(view_server, platform_url)
        bearer_token = c_client.create_egeria_bearer_token(user, password)
        home_metadata_collection = "PostgresContentPack"
        body = {
            "class": "FindDigitalResourceOriginProperties",
            "metadataElementTypeName": "ValidValueDefinition",
            "otherOriginValues": {"homeMetadataCollectionName": home_metadata_collection}
        }
        response = c_client.get_elements_by_origin(body, output_format="JSON", report_spec="Referenceable")

        if type(response) is list:
            print(f"\n\tElement count is: {len(response)}")
            print_json(data=response)
        elif type(response) is str:
            console.print("\n\n\t Response is: " + response)
        assert True
    except PyegeriaException as e:
        print_basic_exception(e)
        assert False, "Invalid request"
    except ValidationError as e:
        print_validation_error(e)
    except Exception as e:
        console.print_exception(show_locals=True)
        assert False, "Invalid request"
    finally:
        c_client.close_session()


def test_get_elements():
    # metadata_element_type_name = 'CertificationType'
    #
    # metadata_element_type_name = "DeployedDatabaseSchema"
    open_metadata_type_name = "UserIdentity"
    try:
        c_client = ClassificationManager(view_server, platform_url)

        bearer_token = c_client.create_egeria_bearer_token(user, password)
        response = c_client.get_elements(open_metadata_type_name, output_format="DICT", report_spec="Referenceable")

        if type(response) is list:
            print(f"\n\tElement count is: {len(response)}")
            print_json(data=response)
        elif type(response) is str:
            console.print("\n\n\t Response is" + response)
        assert True
    except PyegeriaException as e:
        print_basic_exception(e)
        assert False, "Invalid request"
    except Exception as e:
        console.print_exception(show_locals=True)
        assert False, "Invalid request"
    finally:
        c_client.close_session()


def test_get_elements_by_property_value():
    # metadata_element_type_name = 'Project'
    # property_value = "Campaign:Clinical Trials Management"
    # metadata_element_type_name = "ValidValueDefinition"
    metadata_element_type_name = "Collection"
    # property_value = "Unity Catalog Catalog"
    # property_names = ["name", "qualifiedName"]
    # metadata_element_type_name = "Asset"
    # property_value = "ClinicalTrials@CocoPharmaceuticals:set-up-clinical-trial"
    # property_value = "default"
    # property_names = ["name", "qualifiedName"]
    property_names = ["name", "displayName", 'qualifiedName']
    property_value = "Clinical"
    # property_names = ["anchorGUID"]
    try:
        c_client = ClassificationManager(view_server, platform_url)

        bearer_token = c_client.create_egeria_bearer_token(user, password)
        start_time = time.perf_counter()
        result = c_client.get_elements_by_property_value(property_value, property_names, metadata_element_type_name)
        duration = time.perf_counter() - start_time
        print(f"\n\tDuration was {duration} seconds")
        if type(result) is list:
            print(f"\n\tElement count is: {len(result)}")
            print_json(data=result)
        elif type(result) is str:
            console.print("\n\n\t Response is: " + result)

        assert True

    except (
            PyegeriaException
    ) as e:
        print_basic_exception(e)
        assert False, "Invalid request"
    finally:
        c_client.close_session()


def test_find_elements_by_property_value():
    # metadata_element_type_name = 'Project'
    # property_value = "Campaign:Clinical Trials Management"
    # metadata_element_type_name = "ValidValueDefinition"
    # metadata_element_type_name = None
    # metadata_element_type_name = "ArchiveFile"
    open_metadata_type_name = "UserIdentity"
    # metadata_element_type_name = None
    # property_names = ["name"]
    # property_value = "Set up new clinical trial"
    property_names = ["userId"]
    property_value = "erinoverview"

    try:
        c_client = ClassificationManager(view_server, platform_url)

        bearer_token = c_client.create_egeria_bearer_token(user, password)
        start_time = time.perf_counter()
        result = c_client.find_elements_by_property_value(
            property_value, property_names, open_metadata_type_name,
            output_format="JSON", report_spec="Referenceable"
        )
        duration = time.perf_counter() - start_time
        print(f"\n\tDuration was {duration} seconds")
        if type(result) is list:
            print(f"\n\tElement count is: {len(result)}")
            print_json(data=result)
        elif type(result) is str:
            console.print("\n\n\t Response is: " + result)

        assert True

    except (
            PyegeriaException
    ) as e:
        print_basic_exception(e)
        assert False, "Invalid request"
    finally:
        c_client.close_session()


def test_get_element_by_guid():
    element_guid = '0759666b-5aff-4840-8432-63389bd2326a'
    try:
        c_client = ClassificationManager(view_server, platform_url)

        bearer_token = c_client.create_egeria_bearer_token(user, password)
        start_time = time.perf_counter()
        result = c_client.get_element_by_guid(element_guid)
        duration = time.perf_counter() - start_time
        print(f"\n\tDuration was {duration} seconds")
        if type(result) is dict:
            print_json(data=result)
        elif type(result) is str:
            console.print("\n\n\t Response is: " + result)

        assert True

    except (
            PyegeriaException
    ) as e:
        print_basic_exception(e)
        assert False, "Invalid request"
    finally:
        c_client.close_session()


def test_get_actor_for_guid():
    element_guid = "dcfd7e32-8074-4cdf-bdc5-9a6f28818a9d"
    try:
        c_client = ClassificationManager(view_server, platform_url)

        bearer_token = c_client.create_egeria_bearer_token(user, password)
        start_time = time.perf_counter()
        result = c_client.get_actor_for_guid(element_guid)
        duration = time.perf_counter() - start_time
        print(f"\n\tDuration was {duration} seconds")
        if type(result) is dict:
            print_json(data=result)
        elif type(result) is str:
            console.print("\n\n\t Response is: " + result)

        assert True

    except (
            PyegeriaException) as e:
        print_basic_exception(e)
        console.print_exception(show_locals=True)
        assert False, "Invalid request"
    finally:
        c_client.close_session()


def test_get_guid_for_name():
    open_metadata_type_name = None
    # property_value = "Person:UK:324713"
    # property_value = "simple-metadata-store"
    # property_value = "Sustainability Glossary"
    property_value = "qs-view-server"
    c_client = ClassificationManager(view_server, platform_url)

    bearer_token = c_client.create_egeria_bearer_token(user, password)
    start_time = time.perf_counter()
    result = c_client.get_guid_for_name(property_value)
    duration = time.perf_counter() - start_time
    print(f"\n\tDuration was {duration} seconds")
    if type(result) is list:
        print(f"\n\tElement count is: {len(result)}")
        print_json(data=result)
    elif type(result) is str:
        console.print("\n\n\t Response is " + result)

    assert True


def test_get_element_guid_by_unique_name():
    open_metadata_type_name = None
    # property_value = "Person:UK:324713"
    # property_value = "simple-metadata-store"
    property_value = "FileDirectory:CreateAndSurveyGovernanceActionProcess"

    c_client = ClassificationManager(view_server, platform_url)
    try:
        bearer_token = c_client.create_egeria_bearer_token(user, password)
        start_time = time.perf_counter()
        result = c_client.get_element_guid_by_unique_name(property_value, "qualifiedName")
        duration = time.perf_counter() - start_time
        print(f"\n\tDuration was {duration} seconds")
        if type(result) is list:
            print(f"\n\tElement count is: {len(result)}")
            print_json(data=result)
        elif type(result) is str:
            console.print("\n\n\t Response is " + result)
        assert True
    except PyegeriaException as e:
        print_basic_exception(e)
        assert False, "Invalid request"


def test_get_element_by_unique_name():
    open_metadata_type_name = None
    # property_value = "Person:UK:324713"
    property_value = "FileDirectory:CreateAndSurveyGovernanceActionProcess"

    c_client = ClassificationManager(view_server, platform_url)
    try:
        bearer_token = c_client.create_egeria_bearer_token(user, password)

        start_time = time.perf_counter()
        result = c_client.get_element_by_unique_name(property_value, "qualifiedName", output_format="DICT",
                                                     report_spec="Referenceable")
        duration = time.perf_counter() - start_time
        print(f"\n\tDuration was {duration} seconds")
        if isinstance(result, list | dict):
            print(f"\n\tElement count is: {len(result)}")
            print(json.dumps(result, indent=4))
        elif type(result) is str:
            console.print("\n\n\t Response is " + result)

        assert True
    except (PyegeriaException, PyegeriaConnectionException) as e:
        print_basic_exception(e)
        assert False


def test_get_elements_by_classification():
    # metadata_element_type_name = "Project"
    # metadata_element_type_name = "DeployedDatabaseSchema"
    open_metadata_type_name = "Collection"
    # classification = "GovernanceProject"
    classification = "Folder"
    c_client = ClassificationManager(view_server, platform_url)

    bearer_token = c_client.create_egeria_bearer_token(user, password)
    response = c_client.get_elements_by_classification(
        classification, open_metadata_type_name, output_format="DICT", report_spec="Collections"
    )

    if type(response) is list:
        print("Result = \n")
        print_json(data=response)
    elif type(response) is str:
        console.print("\n\n\t Response is: " + response)

    assert True


def test_get_elements_by_classification_with_property_value():
    # metadata_element_type_name = "Project"
    open_metadata_type_name = None
    classification = "DataSpec"
    # property_value = "Collection"
    # property_names = ["anchorTypeName"]
    property_value = ""
    property_names = [""]
    try:
        c_client = ClassificationManager(view_server, platform_url)

        bearer_token = c_client.create_egeria_bearer_token(user, password)
        result = c_client.get_elements_by_classification_with_property_value(
            classification, property_value, property_names, open_metadata_type_name
        )

        if type(result) is list:
            print_json(data=result)
        elif type(result) is str:
            console.print("\n\n\t Response is: " + result)

        assert True

    except (PyegeriaException) as e:
        print_basic_exception(e)
        console.print_exception(show_locals=True)
        assert False, "Invalid request"
    finally:
        c_client.close_session()


def test_find_elements_by_classification_with_property_value():
    # classification = "GovernanceProject"
    # metadata_element_type_name = "Project"
    # property_value = "Clinical Trials"
    # property_names = ["name", "qualifiedName"]
    #
    classification = "DataSpec"
    # metadata_element_type_name = "DeployedDatabaseSchema"
    open_metadata_type_name = None
    property_value = ""
    property_names = ["displayName"]
    c_client = ClassificationManager(view_server, platform_url)

    bearer_token = c_client.create_egeria_bearer_token(user, password)

    start_time = time.perf_counter()
    response = c_client.find_elements_by_classification_with_property_value(
        classification, property_value, property_names, open_metadata_type_name
    )
    duration = time.perf_counter() - start_time
    print(
        f"\n\tDuration was {duration:.2f} seconds, Type: {type(response)}, Element count is {len(response)}"
    )
    if type(response) is list:
        print_json(data=response)
    elif type(response) is str:
        console.print("\n\n\t Response is: " + response)

    assert True


def test_find_anchored_elements_with_property_value():
    classification = "Anchors"
    open_metadata_type_name = None
    property_value = "PostgreSQL Server"
    # property_names = ["ServerCapability", "anchorTypeName"]
    property_names = ["name"]
    c_client = ClassificationManager(view_server, platform_url)

    bearer_token = c_client.create_egeria_bearer_token(user, password)
    response = c_client.find_elements_by_classification_with_property_value(
        classification, property_value, property_names, open_metadata_type_name
    )

    if type(response) is list:
        print("Response payload is: \n")
        print_json(data=response)
    elif type(response) is str:
        console.print("\n\n\t Response is" + response)

    assert True


def test_get_all_related_elements():
    # metadata_element_type_name = 'Project'
    open_metadata_type_name = None
    c_client = ClassificationManager(view_server, platform_url)
    # element_guid = "d156faa6-90cf-4be8-b3c1-c002f3e9a0e5" # branch database
    element_guid = "da0442bf-818f-406b-99dc-83b72605cc98"
    # element_guid = "8b9cce34-ff42-4f9d-b4b3-6317c8a767c3"  # Retail schema
    bearer_token = c_client.create_egeria_bearer_token(user, password)
    response = c_client.get_related_elements(
        element_guid, None, open_metadata_type_name
    )

    if type(response) is list:
        print(f"\n\tElement count is: {len(response)}")
        print_json(data=response)
    elif type(response) is str:
        console.print("\n\n\t Response is" + response)

    assert True


def test_get_related_elements():
    # metadata_element_type_name = 'CertificationType'
    element_guid = '2ce8d10d-08a2-4fca-b0c2-1d5a335d00fc'
    # metadata_element_type_name = "Organization"
    # metadata_element_type_name = "CSVFile"
    # metadata_element_type_name = "InformationSupplyChain"
    open_metadata_type_name = None
    # element_guid = "8dca6e76-d454-4344-9c93-faa837a1a898"
    # relationship_type = "DataContentForDataSet"
    relationship_type = "ResourceList"
    # relationship_type = "InformationSupplyChainComposition"
    c_client = ClassificationManager(view_server, platform_url)
    try:
        bearer_token = c_client.create_egeria_bearer_token(user, password)
        response = c_client.get_related_elements(
            element_guid, relationship_type, open_metadata_type_name
        )

        if type(response) is list:
            print_json(data=response)
        elif type(response) is str:
            console.print("\n\n\t Response is:\n " + response)

        assert True
    except PyegeriaException as e:
        print_basic_exception(e)
        assert False, "Invalid request"


def test_get_related_elements_with_property_value():
    # metadata_element_type_name = 'Project'
    open_metadata_type_name = None
    relationship_type = "ResourceList"
    property_value = "Catalog Resource"
    property_names = [
        "resourceUse",
    ]
    element_guid = '2ce8d10d-08a2-4fca-b0c2-1d5a335d00fc'
    try:
        c_client = ClassificationManager(view_server, platform_url)

        bearer_token = c_client.create_egeria_bearer_token(user, password)
        result = c_client.get_related_elements_with_property_value(
            element_guid,
            relationship_type,
            property_value,
            property_names,
            open_metadata_type_name,
        )

        if type(result) is list:
            print_json(data=result)
        elif type(result) is str:
            print("\n\n\t Response is: " + result)
        else:
            print(f"type is: {type(result)}")

        assert True

    except (
            PyegeriaException) as e:
        print_basic_exception(e)
        console.print_exception(show_locals=True)
        assert False, "Invalid request"
    finally:
        c_client.close_session()


def test_find_related_elements_with_property_value():
    # metadata_element_type_name = 'Project'
    open_metadata_type_name = None
    property_value = "Clinical Trials Management"
    property_names = ["name", "qualifiedName"]
    # property_value = "Partner"
    # property_names = ["teamType"]

    c_client = ClassificationManager(view_server, platform_url)

    bearer_token = c_client.create_egeria_bearer_token(user, password)
    response = c_client.find_related_elements_with_property_value(
        element_guid,
        relationship_type,
        property_value,
        property_names,
        open_metadata_type_name,
    )

    if type(response) is list:
        print_json(data=response)
    elif type(response) is str:
        console.print("\n\n\t Response is " + response)

    assert True


def test_get_relationships():
    c_client = ClassificationManager(view_server, platform_url)

    bearer_token = c_client.create_egeria_bearer_token(user, password)
    relationship_type = "Certification"
    response = c_client.get_relationships(relationship_type)

    if type(response) is list:
        print(f"\n\tElement count is: {len(response)}")
        print_json(data=response)
    elif type(response) is str:
        console.print("\n\n\t Response is: " + response)

    assert True


def test_get_relationships_with_property_value():
    property_value = "Organization:Hampton Hospital"
    property_names = ["name", "qualifiedName"]
    try:
        c_client = ClassificationManager(view_server, platform_url)

        bearer_token = c_client.create_egeria_bearer_token(user, password)
        result = c_client.get_relationships_with_property_value(
            relationship_type, property_value, property_names
        )

        if type(result) is list:
            print_json(data=result)
        elif type(result) is str:
            print("\n\n\t Response is: " + result)
        else:
            print(f"type is: {type(result)}")

        assert True

    except (
            PyegeriaException) as e:
        print_basic_exception(e)
        console.print_exception(show_locals=True)
        assert False, "Invalid request"
    finally:
        c_client.close_session()


def test_find_relationships_with_property_value():
    property_value = "Clinical Trials"
    property_names = ["name", "qualifiedName"]
    c_client = ClassificationManager(view_server, platform_url)

    bearer_token = c_client.create_egeria_bearer_token(user, password)
    response = c_client.find_relationships_with_property_value(
        relationship_type, property_value, property_names
    )

    if type(response) is list:
        print_json(data=response)
    elif type(response) is str:
        console.print("\n\n\t Response is " + response)

    assert True


def test_retrieve_instance_for_guid():
    c_client = ClassificationManager(view_server, platform_url)

    bearer_token = c_client.create_egeria_bearer_token(user, password)
    element_guid = "bda24e8a-4798-4cc0-b693-b09c688d5a6f"
    response = c_client.retrieve_instance_for_guid(element_guid)

    if type(response) is dict:
        print_json(data=response)
    elif type(response) is str:
        console.print("\n\n\t Response is " + response)

    assert True


def test_set_criticality_classification():
    # metadata_element_type_name = 'Project'
    element_guid = "9f6f668d-d9b8-44e3-915b-9edeb4c1f8a5"
    body = {
        "class": "NewClassificationRequestBody",
        "Properties": {
            "class": "CriticalityProperties",
            "levelIdentifier": 3,
            "criticality": 3
        }
    }

    try:
        c_client = ClassificationManager(view_server, platform_url)

        bearer_token = c_client.create_egeria_bearer_token(user, password)
        result = c_client.set_criticality_classification(
            element_guid,
            body
        )

        if type(result) is list:
            print_json(data=result)
        elif type(result) is str:
            print("\n\n\t Response is: " + result)
        else:
            print(f"type is: {type(result)}")

        assert True

    except (
            PyegeriaException
    ) as e:
        print_basic_exception(e)
        assert False, "Invalid request"
    finally:
        c_client.close_session()



import json
import types
from json import JSONDecodeError

import pytest
import os

from pydantic import ValidationError

from pyegeria._client_new import Client2
from pyegeria.config import settings
from pyegeria.external_links import ExternalReferences
from pyegeria.logging_configuration import config_logging

from pyegeria._exceptions_new import PyegeriaException, print_exception_response, print_basic_exception, \
    print_validation_error

EGERIA_USER = os.environ.get("EGERIA_USER", "peterprofile")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")


app_config = settings.Environment
config_logging()
view_server: str = app_config.egeria_view_server
view_url: str = app_config.egeria_view_server_url
user: str = EGERIA_USER
user_pass: str = EGERIA_USER_PASSWORD

def test_add_archive_file():
    server_display_name = "qs-metadata-server"

    try:
        client = Client2(view_server, view_url, user, user_pass)
        client.create_egeria_bearer_token()
        archive_file = "content-packs/CocoComboArchive.omarchive"
        response = client.add_archive_file(archive_file, display_name = "qs-metadata-store")
        print(response)
        assert True
    except PyegeriaException as e:
        print_basic_exception(e)
        assert False



def test_add_search_keyword():
    keyword = "Sentinel"
    element_guid = "a3442fd4-7e23-4778-b139-9739172fdcd7"

    try:
        client = Client2(view_server, view_url, user, user_pass)
        client.create_egeria_bearer_token()

        response = client.add_search_keyword_to_element(element_guid, keyword)
        print(response)
        assert True
    except PyegeriaException as e:
        print_basic_exception(e)
        assert False

def test_find_search_keyword() :
    keyword = "*"
    try:
        client = Client2(view_server, view_url, user, user_pass)
        client.create_egeria_bearer_token()
        response = client.find_search_keywords(keyword, output_format="DICT", report_spec = "Search-Keywords")
        if isinstance(response, dict | list):
            print(json.dumps(response, indent = 2))
        else:
            print(response)
        assert True
    except PyegeriaException as e:
        print_basic_exception(e)
        assert False


def test_remove_search_keyword() :
    keyword_guid = "49cedf27-3aa3-4522-b4b9-7764bac99a3d"
    try:
        client = Client2(view_server, view_url, user, user_pass)
        client.create_egeria_bearer_token()
        client.remove_search_keyword(keyword_guid)

        assert True
    except (PyegeriaException, JSONDecodeError) as e:
        print_basic_exception(e)
        assert False

def test_get_user_guid() :
    user_id = "peterprofile"
    try:
        client = Client2(view_server, view_url, user, user_pass)
        client.create_egeria_bearer_token()
        response = client.get_elements_by_property_value(user_id,['userid', 'displayName','fullName'],None)
        if isinstance(response, dict | list):
            print(json.dumps(response, indent = 2))
        assert True
    except (PyegeriaException, JSONDecodeError) as e:
        print_basic_exception(e)
        assert False

def test_add_journal_entry():
    element_guid = "1a16188a-9cba-4c86-835b-e223e97d22bb" # Milvus
    element_qn = "SolutionComponent::Milvus::V1.0"
    note_log_display_name = "Milvus-NoteLog"
    journal_entry_display_name = "test-journal-entry"
    journal_entry_description = "First Journal Entry"

    try:
        client = Client2(view_server, view_url, user, user_pass)
        client.create_egeria_bearer_token()
        response = client.add_journal_entry(element_qn = element_qn,
                                            note_log_display_name=note_log_display_name,
                                            journal_entry_display_name=journal_entry_display_name,
                                            journal_entry_description=journal_entry_description,)
        print("\n\n" + response)
        assert True
    except (PyegeriaException, JSONDecodeError) as e:
        print_basic_exception(e)
        assert False

def test_find_note_logs():
    try:
        client = Client2(view_server, view_url, user, user_pass)
        client.create_egeria_bearer_token()
        search_string = "*"
        output_format = "DICT"
        report_spec = "Referenceables"
        response = client.find_note_logs(search_string, output_format = output_format, report_spec=report_spec)
        if isinstance(response, dict | list):
            print(json.dumps(response, indent = 2))
        else:
            print(response)
    except (PyegeriaException, JSONDecodeError) as e:
        print_basic_exception(e)
        assert False

def test_remove_note_log() :
    note_log_guid = "5bcc6c62-b9aa-4f56-9d86-4c2bd63eb7f8"
    try:
        client = Client2(view_server, view_url, user, user_pass)
        client.create_egeria_bearer_token()
        client.remove_note_log(note_log_guid)

        assert True
    except (PyegeriaException, JSONDecodeError) as e:
        print_basic_exception(e)
        assert False

def test_find_notes():
    try:
        client = Client2(view_server, view_url, user, user_pass)
        client.create_egeria_bearer_token()
        search_string = "Milvus"
        output_format = "DICT"
        report_spec = "Referenceables"
        response = client.find_notes(search_string, output_format = output_format, report_spec=report_spec)
        if isinstance(response, dict | list):
            print(json.dumps(response, indent = 2))
        else:
            print(response)
    except (PyegeriaException, JSONDecodeError) as e:
        print_basic_exception(e)
        assert False
    except ValidationError as e:
        print_validation_error(e)
        assert False

def test_get_notes_for_note_log():
    try:
        client = Client2(view_server, view_url, user, user_pass)
        client.create_egeria_bearer_token()
        note_log_guid = "5c01cab6-3e57-4f74-b2b2-2438cf36415b"
        output_format = "JSON"
        report_spec = "Referenceables"
        response = client.get_notes_for_note_log(note_log_guid, output_format = output_format, report_spec = report_spec)
        if isinstance(response, dict | list):
            print(json.dumps(response, indent = 2))
        else:
            print(response)
    except (PyegeriaException, JSONDecodeError) as e:
        print_basic_exception(e)
        assert False
